@@ -383,6 +383,9 @@ func (service *serviceType) handleIncomingPacket(bytes []byte) {
 					if value, ok := request["certfile"]; ok {
 						options.Certfile = value.(string)
 					}
+					if value, ok := request["fallback"]; ok {
+						options.Fallback = value.(string)
+					}
 					if request["onRequest"].(bool) {
 						options.OnRequest = func(args api.ServeOnRequestArgs) {
 							// This could potentially be called after we return from
@@ -786,6 +789,8 @@ func resolveKindToString(kind api.ResolveKind) string {
 	// CSS
 	case api.ResolveCSSImportRule:
 		return "import-rule"
+	case api.ResolveCSSComposesFrom:
+		return "composes-from"
 	case api.ResolveCSSURLToken:
 		return "url-token"
 
@@ -812,6 +817,8 @@ func stringToResolveKind(kind string) (api.ResolveKind, bool) {
 	// CSS
 	case "import-rule":
 		return api.ResolveCSSImportRule, true
+	case "composes-from":
+		return api.ResolveCSSComposesFrom, true
 	case "url-token":
 		return api.ResolveCSSURLToken, true
 	}
@@ -1257,6 +1264,7 @@ func encodeOutputFiles(outputFiles []api.OutputFile) []interface{} {
 		values[i] = value
 		value["path"] = outputFile.Path
 		value["contents"] = outputFile.Contents
+		value["hash"] = outputFile.Hash
 	}
 	return values
 }
