@@ -282,7 +282,7 @@ type Lexer struct {
 	Token                           T
 	ts                              config.TSOptions
 	HasNewlineBefore                bool
-	HasPureCommentBefore            bool
+	HasCommentBefore                CommentBefore
 	IsLegacyOctalLiteral            bool
 	PrevTokenWasAwaitKeyword        bool
 	rescanCloseBraceAsTemplateToken bool
@@ -292,6 +292,14 @@ type Lexer struct {
 	IsLogDisabled bool
 }
 
+type CommentBefore uint8
+
+const (
+	PureCommentBefore CommentBefore = 1 << iota
+	KeyCommentBefore
+	NoSideEffectsCommentBefore
+)
+
 type LexerPanic struct{}
 
 func NewLexer(log logger.Log, source logger.Source, ts config.TSOptions) Lexer {
@@ -665,9 +673,9 @@ func RangeOfIdentifier(source logger.Source, loc logger.Loc) logger.Range {
 	return source.RangeOfString(loc)
 }
 
-func RangeOfImportAssertion(source logger.Source, assertion ast.AssertEntry) logger.Range {
-	loc := RangeOfIdentifier(source, assertion.KeyLoc).Loc
-	return logger.Range{Loc: loc, Len: source.RangeOfString(assertion.ValueLoc).End() - loc.Start}
+func RangeOfImportAssertOrWith(source logger.Source, assertOrWith ast.AssertOrWithEntry) logger.Range {
+	loc := RangeOfIdentifier(source, assertOrWith.KeyLoc).Loc
+	return logger.Range{Loc: loc, Len: source.RangeOfString(assertOrWith.ValueLoc).End() - loc.Start}
 }
 
 func (lexer *Lexer) ExpectJSXElementChild(token T) {
@@ -982,7 +990,7 @@ func (lexer *Lexer) NextInsideJSXElement() {
 
 func (lexer *Lexer) Next() {
 	lexer.HasNewlineBefore = lexer.end == 0
-	lexer.HasPureCommentBefore = false
+	lexer.HasCommentBefore = 0
 	lexer.PrevTokenWasAwaitKeyword = false
 	lexer.LegalCommentsBeforeToken = lexer.LegalCommentsBeforeToken[:0]
 	lexer.CommentsBeforeToken = lexer.CommentsBeforeToken[:0]
@@ -2587,7 +2595,13 @@ func (lexer *Lexer) scanCommentText() {
 			rest := text[i+1 : endOfCommentText]
 			if hasPrefixWithWordBoundary(rest, "__PURE__") {
 				omitFromGeneralCommentPreservation = true
-				lexer.HasPureCommentBefore = true
+				lexer.HasCommentBefore |= PureCommentBefore
+			} else if hasPrefixWithWordBoundary(rest, "__KEY__") {
+				omitFromGeneralCommentPreservation = true
+				lexer.HasCommentBefore |= KeyCommentBefore
+			} else if hasPrefixWithWordBoundary(rest, "__NO_SIDE_EFFECTS__") {
+				omitFromGeneralCommentPreservation = true
+				lexer.HasCommentBefore |= NoSideEffectsCommentBefore
 			} else if i == 2 && strings.HasPrefix(rest, " sourceMappingURL=") {
 				if arg, ok := scanForPragmaArg(pragmaNoSpaceFirst, lexer.start+i+1, " sourceMappingURL=", rest); ok {
 					omitFromGeneralCommentPreservation = true
@@ -2599,7 +2613,13 @@ func (lexer *Lexer) scanCommentText() {
 			rest := text[i+1 : endOfCommentText]
 			if hasPrefixWithWordBoundary(rest, "__PURE__") {
 				omitFromGeneralCommentPreservation = true
-				lexer.HasPureCommentBefore = true
+				lexer.HasCommentBefore |= PureCommentBefore
+			} else if hasPrefixWithWordBoundary(rest, "__KEY__") {
+				omitFromGeneralCommentPreservation = true
+				lexer.HasCommentBefore |= KeyCommentBefore
+			} else if hasPrefixWithWordBoundary(rest, "__NO_SIDE_EFFECTS__") {
+				omitFromGeneralCommentPreservation = true
+				lexer.HasCommentBefore |= NoSideEffectsCommentBefore
 			} else if hasPrefixWithWordBoundary(rest, "preserve") || hasPrefixWithWordBoundary(rest, "license") {
 				hasLegalAnnotation = true
 			} else if hasPrefixWithWordBoundary(rest, "jsx") {
